///////////////
// IanniX
///////////////

////////////////////////////
// NxObject.cpp
////////////////////////////

#include "NxObject.h"
#include <iostream>
#include <math.h>

/////////////////////////////////////
// Empty Constructor
/////////////////////////////////////
NxObject::NxObject() 
{}

/////////////////////////////////////
// Default constructor
/////////////////////////////////////
NxObject::NxObject( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor )
{	
	ID = inID;
	customID = incID ;
	groupID = inGroupID;
	name = inName;
	pos = inPos;
	color = inColor;
	selected = false;
	highlighted = false ;
	active = inActive ;
	layer = inLay ;
}

//////////////////////////////////////
// Abstract Destructor
//////////////////////////////////////
NxObject::~NxObject()
{}

/////////////////////////////
// Bounding Box
/////////////////////////////
void NxObject::evalBoundingBox( NxPoint, float )
{}

void NxObject::setBoundingBox( NxPoint inPt1, NxPoint inPt2 )
{
	boundingBox.p1 = inPt1;
	boundingBox.p2 = inPt2;
}
void NxObject::setBox( BBox inBox )
{
	boundingBox = inBox;
}
BBox NxObject::getBoundingBox()
{
	return boundingBox;
}



//////////////////////////////////
// Access Methods
//////////////////////////////////
// SET
void NxObject::setCustomID( int inID )
{
	customID = inID;
}
void NxObject::setGroupID( int inGroupID )
{
	groupID = inGroupID;
}
void NxObject::setName( QString inName )
{
	name = inName;
}
void NxObject::setPosition( NxPoint inPos )
{
	pos = inPos;
}
void NxObject::setColor( Color inColor )
{
	color = inColor;
}
void NxObject::setSelected( bool flag )
{
	selected = flag;
}
void NxObject::setHighlighted( bool flag )
{
	highlighted = flag ;
}
void NxObject::setActive( bool flag )
{
	active = flag ;
}
void NxObject::setLayer( int inLay )
{
	layer = inLay ;
}

// GET
int NxObject::getID()
{
	return ID;
}
int NxObject::getCustomID()
{
	return customID;
}
int NxObject::getGroupID()
{
	return groupID;
}
QString NxObject::getName()
{
	return name;
}
NxPoint	NxObject::getPosition()
{
	return pos;
}
Color NxObject::getColor()
{
	return color;
}
bool NxObject::isSelected()
{
	return selected;
}
bool NxObject::isHighlighted()
{
	return highlighted ;
}
bool NxObject::isActive()
{
	return active ;
}
int NxObject::getLayer()
{
	return layer ;
}
int NxObject::getObjectType()
{
	return objectType;
}
///////////////////////////////////////////////////////////////////////////////
// 					NxCurve
///////////////////////////////////////////////////////////////////////////////
NxCurve::NxCurve() 
{
}

/////////////////////////////////
// Main constructor
/////////////////////////////////
NxCurve::NxCurve(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QVector<NxPoint> inCurves, int inNbPoints, Color inCircleColor, bool inTrans, QString inTexPath, QString inShape, float inRMin, float inRMax, float inScale, float inAngle, int inSpatialPeriod, QString inHost, int inPort )
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	splineGenerated = false ;
	
	spatialPeriod = inSpatialPeriod ; ///// in ms (integer)

	curves			= inCurves;
	nbPoints		= inNbPoints;
	circleColor		= inCircleColor;
	transparency	= inTrans;
	texturePath		= inTexPath;
	shape			= inShape;
	objectType		= Curve;
	showPoints		= false;
	realMin			= inRMin;
	realMax			= inRMax;
	xRange			= 0 ;
	yRange			= fabs(realMax - realMin) ;
	host			= inHost ;
	port			= inPort ;
	angle			= inAngle ;
	scale			= inScale ;

	// Offset Variables
	dX = dY = dZ = 0.0;
	
	// Bounding Box
	if (!curves.isEmpty())
	{
		radius = fabs( curves[inNbPoints-1].x - curves[0].x ) / 2 ;
		xRange = fabs( curves[inNbPoints-1].x - curves[0].x ) ;
		center.x = pos.x + radius ;
		center.y = pos.y ;

		boundingBox.p1.x = inPos.x ;
		boundingBox.p1.y = inPos.y + yRange*scale/2 ;
		boundingBox.p1.z = 0.0;
		boundingBox.p2.x = inPos.x + curves[nbPoints-1].x ;
		boundingBox.p2.y = inPos.y - yRange*scale/2 ;
		boundingBox.p2.z = 0.0;
		
		if(drawType == SplineStrip) splineGenerate() ; // regenerate spline id splines
	}
} 

//////////////////////
// Destructor
//////////////////////
NxCurve::~NxCurve() 
{
}
//////////////////////////////
// NxSpline re-Generation
//////////////////////////////
void NxCurve::splineGenerate()
{
	if(splineGenerated)	delete(splineDef);
	splineDef = new NxSpline( pos, curves.data(), curves.count(), realMin, realMax, scale );
	splineDef->Generate();
	splineGenerated = true ;
}
//////////////////////////////
// Bounding Box
//////////////////////////////
void NxCurve::evalBoundingBox( NxPoint inPos , float )
{	
	// Calculating offsets
	dX = inPos.x - center.x + radius ;
	dY = inPos.y - center.y ;
	dZ = inPos.z - center.z ;
	
	if(drawType == SplineStrip) splineGenerate() ; // only if spline mode
	
	// Updating Circle
	center.x = inPos.x + radius ;
	center.y = inPos.y ;
	center.z = inPos.z ;
	
	boundingBox.p1.x = inPos.x ;
	boundingBox.p1.y = inPos.y + yRange*scale/2 ;
	boundingBox.p1.z = 0.0;
	boundingBox.p2.x = inPos.x + curves[nbPoints-1].x ;
	boundingBox.p2.y = inPos.y - yRange*scale/2 ;
	boundingBox.p2.z = 0.0;
}

/////////////////////////////////
// Access methods
/////////////////////////////////

///////////
// SET
///////////
void NxCurve::setCurve( QVector<NxPoint> inCurve )
{
	curves.clear();
	curves = inCurve;
	
	nbPoints 	= inCurve.count();
	center.x 	= pos.x + ( curves[nbPoints-1].x + curves[0].x ) / 2;
	center.y 	= pos.y ;
	radius 		= fabs( curves[nbPoints-1].x - curves[0].x ) / 2 ;
	xRange		= fabs( curves[nbPoints-1].x - curves[0].x ) ;
	// Re-calculate Bounding Box
	boundingBox.p1.x = center.x - radius;
	boundingBox.p1.y = center.y + yRange*scale/2 ;
	boundingBox.p1.z = 0.0 ;
	boundingBox.p2.x = center.x + radius;
	boundingBox.p2.y = center.y - yRange*scale/2 ;
	boundingBox.p2.z = 0.0 ;
	
	if(drawType == SplineStrip) splineGenerate() ; // only if spline mode
}
bool NxCurve::isInside( float inX, float inY )
{
	bool res = false ;
	if(angle==0) {
		if ( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )
			res = true ;
		}
		else {
			float hh = fabs(boundingBox.p2.y - boundingBox.p1.y) ;
			float curvAngle = angle*PI/180.0 ;
			//printf("Angle : %f\n", curvAngle ) ;
			float newInX = inX - center.x ;
			float newInY = inY - center.y ;
			//printf("// center : %f et %f\n", newInX, newInY ) ;
			float aSin = sin(curvAngle) ;
			float aCos = cos(curvAngle) ;
			//printf("sin / cos : %f et %f\n", aSin, aCos ) ;
			float localInX = aCos*newInX + aSin*newInY ;
			float localInY = -aSin*newInX + aCos*newInY ;
			//printf("contours : %f et %f\n", localInX, localInY ) ;
			//printf("local : %f et %f\n", xRange, hh ) ;
			if( (fabs(localInY)<hh/2.0) && (fabs(localInX)<xRange/2.0) ) res = true ;
		}
	return res ;
}
void NxCurve::setCircleColor( Color inColor )
{
	circleColor = inColor;
}
void NxCurve::setCenter( NxPoint inPoint )
{
	center = inPoint;
}
void NxCurve::setRadius( float inF )
{
	radius = inF;
}
void NxCurve::setTransparency( bool flag )
{
	transparency = flag;
}
void NxCurve::setTexturePath( QString inPath )
{
	texturePath = inPath;
}
void NxCurve::setShape( QString inShape )
{
	shape = inShape;
}
void NxCurve::setPoints( bool flag )
{
	showPoints = flag;
}
float NxCurve::reEvalRealMin( float inMin )
{	
	float rMin = inMin ;
	for(int i=0; i<size(); i++) {
		if (curves[i].y < rMin) rMin = curves[i].y ;
	}
	return rMin ;
}
float NxCurve::reEvalRealMax( float inMax )
{
	float rMax = inMax ;
	for(int i=0; i<size(); i++) {
		if (curves[i].y > rMax) rMax = curves[i].y ;
	}
	return rMax ;
}
void NxCurve::setRealValues( float inMin, float inMax )
{
	realMin = reEvalRealMin( inMin ) ;
	realMax = reEvalRealMax( inMax ) ;
	if(drawType == SplineStrip)
		splineDef->setRealValues( realMin, realMax ) ;
		
	yRange = fabs(realMax - realMin) ;
	evalBoundingBox( pos, 0 ) ;
}
void NxCurve::setRealMin( float inMin )
{
	realMin = reEvalRealMin( inMin ) ;
	if(drawType == SplineStrip) 
		splineDef->setRealMin( realMin ) ;
		
	yRange = fabs(realMax - realMin) ;
	evalBoundingBox( pos, 0 ) ;
}
void NxCurve::setRealMax( float inMax )
{
	realMax = reEvalRealMax( inMax ) ;
	if(drawType == SplineStrip) 
		splineDef->setRealMax( realMax ) ;
		
	yRange = fabs(realMax - realMin) ;
	evalBoundingBox( pos, 0 ) ;
}
void NxCurve::setSplineMinMax()
{
	splineDef->setRealValues( realMin, realMax ) ;
}
void NxCurve::setAngle( float inAngle )
{
	angle = inAngle;
}
void NxCurve::setScale( float inScale )
{
	scale = inScale;
	if(drawType == SplineStrip) splineGenerate() ; // regenerate spline
}
void NxCurve::setHost( QString inHost )
{
	host = inHost ;
}
void NxCurve::setPort( int inPort ) 
{
	port = inPort ;
}
void NxCurve::setSpatialPeriod( int inP )
{
	spatialPeriod = inP ;
}
void NxCurve::setDrawType( int inType )
{
	drawType = inType;
	if (inType == SplineStrip) splineGenerate() ;
}

///////////
// GET 
///////////
QVector<NxPoint> NxCurve::getCurve()
{
	return curves;
}
NxSpline* NxCurve::getSpline()
{
	return splineDef;
}
Color NxCurve::getCircleColor()
{
	return circleColor;
}
NxPoint NxCurve::getCenter()
{
	return center;
}
NxPoint NxCurve::getEnd()
{
	NxPoint res = pos ;
	res.x += 2*radius*cos(angle*PI/180.0) ;
	res.y += 2*radius*sin(angle*PI/180.0) ;
	return res ;
}
float NxCurve::getRadius()
{
	return radius;
}
float NxCurve::getRealMin()
{
	return realMin;
}
float NxCurve::getRealMax()
{
	return realMax;
}
float NxCurve::getXRange()
{
	return xRange;
}
float NxCurve::getYRange()
{
	return yRange;
}
float NxCurve::getScale()
{
	return scale ;
}
float NxCurve::getAngle()
{
	return angle;
}
QString NxCurve::getHost()
{
	return host;
}
int NxCurve::getPort()
{
	return port;
}
int NxCurve::getSpatialPeriod()
{
	return spatialPeriod ;
}
QString NxCurve::getTexturePath()
{
	return texturePath;
}
QString NxCurve::getShape()
{
	return shape;
}
bool NxCurve::displayPoints()
{
	return showPoints;
}
int NxCurve::size()
{
	return nbPoints;
}
bool NxCurve::isTransparent()
{
	return transparency;
}
int NxCurve::getDrawType()
{
	return drawType;
}
float NxCurve::getLinearValue( float inTime )
{
	float res = 0 ; // initialisation
	int tempID = 0 ;
	float x1, x2, y1, y2 ;
	bool valueFound = false ;
	
	if( (inTime<0.0) || (inTime>curves[size()-1].x) ) {
		valueFound = true ;
		res = 0.0 ;
		}
		
	while( !valueFound ) {
		x1 = curves[tempID].x ;
		x2 = curves[tempID+1].x ;
		if( inTime <= x2 ) {
			valueFound = true ;
			y1 = curves[tempID].y ;
			y2 = curves[tempID+1].y ;
			res = (inTime-x1)*(y2-y1)/(x2-x1) + y1 ;
			}
		else tempID++ ;
		}

	return res ;
}

///////////////////////////////////////////////////////////////////////////////
// 	NxTraject
///////////////////////////////////////////////////////////////////////////////

//////////////////////////
// Constructors
//////////////////////////
NxTraject::NxTraject()
	: NxObject()
{}

/////////////////////////////////
// Main constructor
/////////////////////////////////
NxTraject::NxTraject(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor,
				int inType, int inNbCursors, QList<int> inCursors )
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor ) 
{
	type = inType;
	nbCursors = inNbCursors;
	cursors = inCursors;
	objectType = Trajectory;
	dX = dY = dZ = angle = length = 0.0;
}

//////////////////////
// Destructor
//////////////////////
NxTraject::~NxTraject() {}

/////////////////////////////
// Bounding Box
/////////////////////////////
void NxTraject::evalBoundingBox( NxPoint inPos, float inSize )
{
	switch (type)
	{
		case 1: // Circles
			boundingBox.p1.x = inPos.x - circ.radius;
			boundingBox.p1.y = inPos.y + circ.radius;
			boundingBox.p1.z = 0.0;
	
			boundingBox.p2.x = inPos.x + circ.radius;
			boundingBox.p2.y = inPos.y - circ.radius;
			boundingBox.p2.z = 0.0;
			circ.p = inPos;
		break;
		case 2 : // Lines
			dX = inPos.x - initPt.x ;
			dY = inPos.y - initPt.y ;
			dZ = inPos.z - initPt.z ;
			
			initPt.x += dX;
			initPt.y += dY;
			initPt.z += dZ;
			endPt.x += dX;
			endPt.y += dY;
			endPt.z += dZ;
			
			if ( fmod( angle, 90.0 )>10.0 && fmod( angle, 90.0 )<80.0 )
			{						
				boundingBox.p1 = initPt;
				boundingBox.p2 = endPt;	
				//printf("first case\n") ;
			}
			else
			{
				if ( fabs( initPt.x - endPt.x ) > fabs( initPt.y - endPt.y ) ) { //// dY is small
					boundingBox.p1.x = initPt.x ;
					boundingBox.p2.x = endPt.x ;
					if ( initPt.y > endPt.y ) {
						boundingBox.p1.y = initPt.y + inSize ;
						boundingBox.p2.y = endPt.y - inSize ;
					}
					else {
						boundingBox.p1.y = initPt.y - inSize ;
						boundingBox.p2.y = endPt.y + inSize ;
					}
				}
				else { /////// dX is small
					boundingBox.p1.y = initPt.y ;
					boundingBox.p2.y = endPt.y ;
					if ( initPt.x > endPt.x ) {
						boundingBox.p1.x = initPt.x + inSize;
						boundingBox.p2.x = endPt.x - inSize;
					}
					else {
						boundingBox.p1.x = initPt.x - inSize ;
						boundingBox.p2.x = endPt.x + inSize ;
					}

				} 				 
			}				
			dX = inPos.x - initPt.x;
			dY = inPos.y - initPt.y;
			dZ = inPos.z - initPt.z;
		break;
	}
}
bool NxTraject::isInside( float inX, float inY, float inBBsize )
{
	float distte ;
	bool res = false ;
	switch( type) {
		case 1 : // Circle
			//OLD ONE
			/*if ( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )
				res = true ;*/
			distte = sqrt( (pos.x - inX)*(pos.x - inX)+(pos.y - inY)*(pos.y - inY) ) ;
			if( fabs(circ.radius-distte)<=inBBsize ) res = true ;
			return res ;
		break ;
		case 2 : // Line
			switch( orientation ) {
				case Horizontal :
					if( 
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )	)
							res = true ;
				case Vertical :
					if( 
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )	)
							res = true ;
				break ;
				case Both :
					//OLD ONE
					/*if( 
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p1.y && inY <= boundingBox.p2.y ) ) ||
						( ( inX >= boundingBox.p2.x && inX <= boundingBox.p1.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )	)
							res = true ;
					*/
					NxPoint center ;
					center.ID = 0 ;
					center.z = 0 ;
					center.x = (initPt.x + endPt.x) / 2 ;
					center.y = (initPt.y + endPt.y) / 2 ;
					float trajAngle = angle*PI/180.0 ;
					if(endPt.y<initPt.y) trajAngle = PI - trajAngle ;
					//printf("Angle : %f\n", trajAngle ) ;
					// we change the reference...
					float newInX = inX - center.x ;
					float newInY = inY - center.y ;
					//printf("// center : %f et %f\n", newInX, newInY ) ;
					float aSin = sin(trajAngle) ;
					float aCos = cos(trajAngle) ;
					//printf("sin / cos : %f et %f\n", aSin, aCos ) ;
					float localInX = aCos*newInX + aSin*newInY ;
					float localInY = -aSin*newInX + aCos*newInY ;
					//printf("local : %f et %f\n", localInX, localInY ) ;
					//printf("BB et lng : %f et %f\n", inBBsize, length ) ;
					if( fabs(localInY)<inBBsize && fabs(localInX)<length/2.0 ) res = true ;
				break ;
			}
		break ;
	}
	return res ;
}
///////////
// SET
///////////
void NxTraject::setCircleDef( Circle inC )
{
	circ = inC;
} 
void NxTraject::setLength( float inLength )
{
	if( type == 2 ) { // Line
		float oldLength = this->getLength() ;
		NxPoint Pt1 = initPt ;
		NxPoint Pt2 = endPt ;
		float dx = Pt2.x - Pt1.x ;
		float dy = Pt2.y - Pt1.y ; 
		float a = oldLength / fabs(inLength) ;
		Pt2.x = Pt1.x + dx / a ;
		Pt2.y = Pt1.y + dy / a ;
		this->setLineDef( Pt1, Pt2 ) ;
	}
	else { // Circle
		circ.radius = fabs(inLength) / (2*PI) ;
	}
}
void NxTraject::setLineDef( NxPoint inInit, NxPoint inEnd )
{
	initPt = inInit;
	endPt = inEnd;
	length = sqrt( pow( endPt.x - initPt.x, 2.0 ) + pow( endPt.y - initPt.y, 2.0 ) );
	
	if(length==0 || fabs((endPt.x - initPt.x)-length)<0.0001) angle = 0.0 ; 
	else angle = acos( (endPt.x - initPt.x)/length )*180.0/PI ;
}
void NxTraject::setInitialPoint( NxPoint inPt )
{
	initPt = inPt;
	length = sqrt( pow( endPt.x - initPt.x, 2.0 ) + pow( endPt.y - initPt.y, 2.0 ) );
	if(length==0 || fabs((endPt.x - initPt.x)-length)<0.0001) angle = 0.0 ; 
	else angle = acos( (endPt.x - initPt.x)/length )*180.0/PI ;
}
void NxTraject::setFinalPoint( NxPoint inPt )
{
	endPt	= inPt;
	length = sqrt( pow( endPt.x - initPt.x, 2.0 ) + pow( endPt.y - initPt.y, 2.0 ) );
	if(length==0 || fabs((endPt.x - initPt.x)-length)<0.0001) angle = 0.0 ; 
	else angle = acos( (endPt.x - initPt.x)/length )*180.0/PI ;
}
void NxTraject::setType (int inType )
{
	type = inType;
}
void NxTraject::setCursorIDs( QList<int> inIDs )
{
	cursors = inIDs;
}
void NxTraject::setOrientation( int inOrient )
{
	orientation = inOrient;
}
void NxTraject::addChild( int ID )
{
	nbCursors++;
	cursors << ID;
}
void NxTraject::removeChild( int ID )
{
	nbCursors--;
	cursors.removeAll(ID);
}


///////////
// GET
///////////
Circle NxTraject::getCircle()
{
	return circ;
}
int NxTraject::getType()
{
	return type;
}
QList<int> NxTraject::getCursorIDs()
{
	return cursors;
}
int NxTraject::getNbCursors()
{
	return nbCursors;
}
NxPoint NxTraject::getInitialPoint()
{
	return initPt;
}
NxPoint NxTraject::getFinalPoint()
{
	return endPt;
}
float NxTraject::getLength()
{
	if( type == 1 ) // Circle
		return circ.radius*2*PI ;
	else // Line
		return length;
}
float NxTraject::getAngle()
{
	if( type == 1 ) // Circle
		return 0 ;
	else // Line
		return angle ;
}
int NxTraject::getOrientation()
{
	return orientation;
}

///////////////////////////////////////////////////////////////////////////////
//					NxTrigger
///////////////////////////////////////////////////////////////////////////////

/////////////////////////
// Constructor
/////////////////////////
NxTrigger::NxTrigger(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QString inHost, int inPort, int inIntValue)
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	color = inColor ;
	host = inHost;
	port = inPort;
	value = inIntValue ;
	objectType = Trigger;
		
	evalBoundingBox( inPos, 1.0 );
}

NxTrigger::~NxTrigger() {}

void NxTrigger::evalBoundingBox( NxPoint inPos, float inSize)
{
	// Creating Bounding Box
	boundingBox.p1.x = inPos.x - inSize;
	boundingBox.p1.y = inPos.y + inSize;
	boundingBox.p1.z = 0.0;
	
	boundingBox.p2.x = inPos.x + inSize;
	boundingBox.p2.y = inPos.y - inSize;
	boundingBox.p2.z = 0.0;
}
bool NxTrigger::isInside( float inX, float inY )
{
	bool res = false ;
	if ( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )
		res = true ;
	return res ;
}
//////////////////////
// SET / GET
//////////////////////
void NxTrigger::setHost( QString inHost )
{
	host = inHost ;
}
void NxTrigger::setPort( int inPort ) 
{
	port = inPort ;
}
void NxTrigger::setValue( int inValue ) 
{
	value = inValue ;
}
QString NxTrigger::getHost()
{
	return host;
}
int NxTrigger::getPort()
{
	return port;
}
int NxTrigger::getValue()
{
	return value;
}


///////////////////////////////////////////////////////////////////////////////
// 					NxCursor
///////////////////////////////////////////////////////////////////////////////
 
/////////////////////////
// Constructor
/////////////////////////
NxCursor::NxCursor( int inID, int incID, int inGroupID, int inLay, NxTraject* inParentTraj, QString inName, bool inActive, NxPoint inPos, Color inColor, bool inPlayF, bool inForward, int inMode, 
				State* inSpeedDef, State* inWidthDef, int inNSpeed, int inNWidth, float inWidth, float inInitOffset, float inFinalOffset)
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	parentTrajectory = inParentTraj ;
	forward = inForward ;
	mode = inMode;
	nbSpeed = inNSpeed;
	nbWidth = inNWidth;
	speedDef = inSpeedDef;
	widthDef = inWidthDef;
	objectType = Cursor;
	speed = 1.0 ;
	width = inWidth;
	initOffset = inInitOffset ;
	finalOffset = inFinalOffset ;
	
	playFinished = inPlayF ;
	forwardChanged = false ;
	
	if( parentTrajectory->getLength()==finalOffset ) customFinalOffset = false ;
	else customFinalOffset = true ;
	
	reEvalOffsets() ; // reajust offsets / length of the trajectory
	
	if(forward) offset = initOffset ;
		else offset = parentTrajectory->getLength() - initOffset ;

	NxPoint p1 = parentTrajectory->getInitialPoint();
	NxPoint p2 = parentTrajectory->getFinalPoint();
	Circle myCircle = parentTrajectory->getCircle();

	switch( parentTrajectory->getType() )
	{
		case 1 : // Circles
			length	= 2 * PI * myCircle.radius;
			angle	= 0.0 ;
		break;
		case 2 : // Lines
			length	= sqrt( pow( p2.x - p1.x, 2.0 ) + pow( p2.y - p1.y, 2.0 ) );
			if(length==0 || fabs((p2.x - p1.x)-length)<0.0001) angle = 0.0 ; 
			else angle = acos( (p2.x - p1.x)/length )*180.0/PI ;
			
			if ( ( p1.x > p2.x && !( p1.y < p2.y ) ) || p1.y > p2.y )
				angle = -angle;
		break;
	}	
	evalBoundingBox( inPos, 1.0 );
}

//////////////////////
// Destructor
//////////////////////
NxCursor::~NxCursor()
{
	delete &speedDef;
	delete &widthDef;
	delete &parentID;
}

/////////////////////////////
// Bounding Box
/////////////////////////////
void NxCursor::evalBoundingBox( NxPoint inPos, float inSize )
{
	boundingBox.p1.x = inPos.x - inSize;
	boundingBox.p1.y = inPos.y + 2*inSize;
	boundingBox.p1.z = inPos.z;
	
	boundingBox.p2.x = inPos.x + inSize;
	boundingBox.p2.y = inPos.y - 2*inSize;
	boundingBox.p2.z = inPos.z;
	
	NxPoint p1 = parentTrajectory->getInitialPoint();
	NxPoint p2 = parentTrajectory->getFinalPoint();
	Circle myCircle = parentTrajectory->getCircle();
	
	switch( parentTrajectory->getType() )
	{
		case 1 :
			length	= 2 * PI * myCircle.radius ;
			angle	= 0.0 ;
		break;
		case 2 :
			length	= sqrt( pow( p2.x - p1.x, 2.0 ) + pow( p2.y - p1.y, 2.0 ) );
			
			if(length==0 || fabs((p2.x - p1.x)-length)<0.0001) angle = 0.0 ; 
			else angle = acos( (p2.x - p1.x)/length )*180.0/PI ;
			
			if ( ( p1.x > p2.x && !( p1.y < p2.y ) ) || p1.y > p2.y )
				angle = -angle;
		break;
	}		
}
bool NxCursor::isInside( float inX, float inY )
{
	bool res = false ;
	if (	(( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y )) ||
			(( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY <= boundingBox.p2.y && inY >= boundingBox.p1.y )) ||
			(( inX <= boundingBox.p1.x && inX >= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y )) || 
			(( inX <= boundingBox.p1.x && inX >= boundingBox.p2.x ) && ( inY <= boundingBox.p2.y && inY >= boundingBox.p1.y )) )
		res = true ;

	return res ;
}
///////////
// SET
///////////
void NxCursor::changeParent( int newID )
{
	parentID = newID;
}
void NxCursor::setSpeedDef( State* inSpeedDef )
{
	speedDef = inSpeedDef;
}
void NxCursor::setAngle( float inAngle )
{
	angle = inAngle;
}
void NxCursor::setSpeed( float inSpeed )
{
	speed = inSpeed;
}
void NxCursor::setWidth( float inWidth )
{
	width = inWidth;
}
void NxCursor::setWidthDef( State* inWidthDef )
{
	widthDef = inWidthDef;
}
void NxCursor::setForward( bool inForward )
{
	if(forward!=inForward) { // Forward mode changed
		forwardChanged = true ;
		float len = ((NxTraject*)parentTrajectory)->getLength() ;
		float newInitO = len - finalOffset ;
		float newFinalO = len - initOffset ;
		setInitOffset( newInitO ) ;
		setFinalOffset( newFinalO ) ;
	}
	forward = inForward ;
}
void NxCursor::setMode( int inMode )
{
	mode = inMode;
}
void NxCursor::setChangedDirection( bool chooser ) 
{
	forwardChanged = chooser ;
}
void NxCursor::setInit( NxPoint inPt )
{
	init = inPt;
}
void NxCursor::setOffset( float inOffset )
{
	offset = inOffset;
	
	if( parentTrajectory->getType()==1 ) { // Circle
		if(length!=0) angle = inOffset * 360.0 / length ;
		else angle = 0 ;
	}
}
void NxCursor::setInitOffset( float inOffset )
{
	initOffset = inOffset;
}
void NxCursor::setFinalOffset( float inOffset )
{
	finalOffset = inOffset;
	if( parentTrajectory->getLength()==finalOffset ) customFinalOffset = false ;
	else customFinalOffset = true ;
}
void NxCursor::reEvalOffsets() // not very beautiful..
{
	float len = ((NxTraject*)parentTrajectory)->getLength() ;
	if (initOffset>=len)	initOffset = 0.0 ;
	if (initOffset<0)		initOffset = 0.0 ;
	if (finalOffset>=len)	customFinalOffset = false ;
	if (finalOffset<0)		customFinalOffset = false ;
	if(!customFinalOffset)		finalOffset = len ;
	if (initOffset>=finalOffset) {
		float temp = initOffset ;
		initOffset = finalOffset ;
		finalOffset = temp ;
	}
	if(offset>finalOffset)	offset = finalOffset ;
	if(offset<initOffset)	offset = initOffset ;
	//printf("init: %f final : %f \n", initOffset, finalOffset) ;
	//if(customFinalOffset) printf("i have a custom final offset!!\n") ;
}
void NxCursor::hardOffsetsReset()
{
	initOffset = 0.0 ;
	finalOffset = ((NxTraject*)parentTrajectory)->getLength() ;
	customFinalOffset = false ;
}
void NxCursor::setPlayFinished( bool chooser )
{
	playFinished = chooser ;
}
void NxCursor::setParentTrajectory( NxTraject* newParent )
{
	parentTrajectory = newParent;
}


///////////
// GET
///////////
int NxCursor::getParentID()
{
	return parentID;
}
State* NxCursor::getSpeedCurve() 
{	
	return speedDef;
}
State* NxCursor::getWidthCurve()
{
	return widthDef;
}
bool NxCursor::getForward()
{
	return forward ;
}
int NxCursor::getMode()
{
	return mode ;
}
bool NxCursor::hasChangedDirection() 
{
	return forwardChanged ;
}
int NxCursor::getNbSpeed()
{
	return nbSpeed;
}

///////////////////////////////////////////////
// Speed / Width Evaluator
///////////////////////////////////////////////
float NxCursor::getSpeedWidth( float inTime, int chooser )
{
	// Local variables
	int nbStates;
	State* timeline = NULL;
	int i = 0;
	int millisTime = (int)( inTime*1000 );
	float result = 0.0;
	
	// Return 0.0 if no timeline def -> temp
	if ( (nbWidth == 0) && (chooser == 0) )
		return 0.0;
		
	if ( (nbSpeed == 0) && (chooser == 1) )
		return 0.0;
		
	// Choose between speed 1 / width 0
	if (chooser == 0)
	{
		timeline = widthDef;
		nbStates = nbWidth;
		result = width;
	}
	else
	{
		timeline = speedDef;
		nbStates = nbSpeed;
		result = speed;
	}
		float globalTime = timeline[0].duration;
		
	// Searching State
	while( i<nbStates && globalTime < millisTime )
	{
		i++;
		globalTime += timeline[i].duration;
	}
	
	///////////////////////
	// Evaluating 
	///////////////////////
	if (timeline[i].type == 1)
		result = (timeline[i].coeff * millisTime) / 1000.0;
		
	if (timeline[i].type == 2)
		result += cos(millisTime) * timeline[i].scale; 
	
	if (timeline[i].type == 3)
		result += sin(millisTime) * timeline[i].scale;
	
	if (timeline[i].type == 4)
		result += log( millisTime - (globalTime - timeline[i].duration) ) * timeline[i].scale;
	
	return result;
}

int NxCursor::getNbWidth()
{
	return nbWidth;
}
float NxCursor::getAngle()
{
	return angle;
}
float NxCursor::getLength()
{
	return length;
}
float NxCursor::getWidth()
{
	return width;
}
float NxCursor::getSpeed()
{
	return speed;
}
NxPoint NxCursor::getInit()
{
	return init;
}
float NxCursor::getOffset()
{
	return offset;
}
float NxCursor::getInitOffset()
{
	return initOffset;
}
float NxCursor::getFinalOffset()
{
	return finalOffset;
}
bool NxCursor::hasCustomFinalOffset()
{
	return customFinalOffset;
}
bool NxCursor::getPlayFinished() 
{
	return playFinished ;	
}
NxTraject* NxCursor::getParentTrajectory()
{
	return parentTrajectory;
}

