///////////////
// IanniX
///////////////

////////////////////////////
// NxTime.cpp
////////////////////////////

#include "NxTime.h"

NxTime::NxTime( QWidget* parent, int chooser )
    : QDialog( parent )
{
	this->setWindowModality( Qt::ApplicationModal ) ;
	
	if(chooser==0) {
		this->setWindowTitle("Width editor") ;
		editorType = 0 ;
	}
	else {
		this->setWindowTitle("Speed editor") ;
		editorType = 1 ;
	}
		
	// Font Setting
	//QFont f( "Helvetica", 10);
	//setFont( f );
	
	// Setting parent
	parentBox = (NxGLScore*) parent;
	
	// Control variables
	nbStates = 0 ;
	tmpStates = (State*) calloc( 1, sizeof( State ) ) ;
	 
	QVBoxLayout *mainLayout = new QVBoxLayout(this) ; 
	// 
	QHBoxLayout *buttons = new QHBoxLayout ;
	QHBoxLayout *states = new QHBoxLayout ;
	QHBoxLayout *painter = new QHBoxLayout ;
	painter->setAlignment( Qt::AlignHCenter );
 	/////// States
	statesTable = new QTableWidget( 0, 7, this ) ;
	statesTable->setHorizontalHeaderLabels( QStringList() << "In" << "Out" << "Type" << "Frequency" << "Coeff" << "Scale" << "Duration"  );
	statesTable->setMinimumSize( 700, 300 );
	states->addWidget( statesTable ) ;

	/////// Buttons
	addButton = new QPushButton("Add State", this) ;
	removeButton = new QPushButton("Remove State", this) ;
  	applyButton = new QPushButton("Apply", this) ;
	buttons->addWidget(addButton) ;	
	buttons->addWidget(removeButton) ;
  	buttons->addWidget(applyButton) ;
	
	// NxPainter Declaration
	timePainter = new NxPainter( this );
	timePainter->setMinimumSize( 700, 350 );
	timePainter->setMaximumSize( 700, 350 );
	painter->addWidget( timePainter ) ; 
		
	// Main Layout
	mainLayout->addLayout( states );
	mainLayout->addLayout( buttons );
	mainLayout->addLayout( painter );

	this->setLayout( mainLayout ) ;
	
	connect( addButton, SIGNAL( clicked() ), this, SLOT( addLine() ) );
	connect( removeButton, SIGNAL( clicked() ), this, SLOT( removeLine() ) );
	connect( applyButton, SIGNAL( clicked() ), this, SLOT( assignTimeline() ) );
	connect( statesTable, SIGNAL(cellChanged(int, int)), this, SLOT(updateState(int, int)) );
}

NxTime::~NxTime()
{
    // no need to delete child widgets, Qt does it all for us
}

////////////////////////////////////
// adds a line
////////////////////////////////////
void NxTime::addLine()
{
	nbStates++ ;	
	// Dynamic memory allocation
	tmpStates = (State*) realloc (tmpStates, (nbStates+1) * sizeof ( State ) );	
	statesTable->setRowCount( nbStates ) ;
	
	// IN
	statesTable->setItem( nbStates-1, 0, new QTableWidgetItem(QString("unused")) );
	// OUT
	statesTable->setItem( nbStates-1, 1, new QTableWidgetItem(QString("unused")) );
	// TYPE
	QStringList combList;
	combList << "Constant" << "Linear" << "Sin" << "Cos" << "Log"  ;
	QComboBox *combo = new QComboBox;
	combo->addItems( combList ) ;
	combo->setCurrentIndex(0) ;
	statesTable->setCellWidget( nbStates-1, 2, combo );
	connect( combo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateStateTypes()) ) ;
	// FREQ
	statesTable->setItem( nbStates-1, 3, new QTableWidgetItem(QString("7")) );
	// COEFF
	statesTable->setItem( nbStates-1, 4, new QTableWidgetItem(QString("1")) );	
	// SCALE
	statesTable->setItem( nbStates-1, 5, new QTableWidgetItem(QString("100")) );
	// DURATION
	statesTable->setItem( nbStates-1, 6, new QTableWidgetItem(QString("1000")) );
}
////////////////////////////////////
// removes the last line
////////////////////////////////////
void NxTime::removeLine()
{
	nbStates-- ;	
	statesTable->setRowCount( nbStates ) ;
	updatePainter() ;
}
//////// Table UPDATES ///////
void NxTime::updateStateTypes()
{
	QString t ;
	for(int i=0 ; i<nbStates ; i++)
		tmpStates[i].type = ( (QComboBox*)statesTable->cellWidget(i,2) )->currentIndex() ;
	updatePainter() ;
}
void NxTime::updateState( int inRow, int inCol )
{
	QString str ;
	str = statesTable->item( inRow, inCol )->text();	
	switch (inCol)
	{
		case 3 : // frequency
			tmpStates[inRow].frequency = str.toFloat() ;
		break;		
		case 4 : // coeff
			tmpStates[inRow].coeff = str.toFloat() ;		
		break;	
		case 5 : // scale
			tmpStates[inRow].scale = str.toFloat() ;		
		break;
		case 6 : // duration
			tmpStates[inRow].duration = str.toFloat() ;
		break;
	}
	updatePainter() ;
}

void NxTime::updatePainter()
{	
	// TO DO : test to have good values ...

	// Updating Painter Object
	timePainter->setStates( tmpStates, nbStates );
	timePainter->repaint();
}

void NxTime::assignTimeline()
{
	// editorType :   0=Width / 1=Speed
	parentBox->setTimeline( tmpStates, editorType ) ;
	this->close() ;
}

