///////////////
// IanniX
///////////////

////////////////////////////
// NxParser.cpp
////////////////////////////

#include "NxParser.h"

NxParser::NxParser(QString inFile)
{
	// Reading XML Document
	QFile newFile(inFile);
	read(&newFile);
}

void NxParser::read(QIODevice *file)
{
	if (!document.setContent(file, true)) 
		printf("AARGH : XML loading failed !!! \n");
}


// ====================
// ### PARSE CURVES ###
// ====================
QMap<int, NxCurve*> NxParser::parseCurves()
{
	// Curves data
  	int 				ID,custID, groupID, layer, nbPoints, dType, spatialPeriod, port ;
  	float 				realMin, realMax, scale, angle ;
	bool				act ;
	QString				name, transparency, texture_path, shape, host;
  	Color 				c1, c2;
  	QVector<NxPoint> 	points;
  	QList<Color>		colors;
	NxPoint				curvePos ;
	
  	// Parsing NxCurve nodes
	QDomNodeList nodes = document.elementsByTagName("NxCurve");
	
  	for( int i=0; i<nodes.count(); i++)
	{
		QDomElement cNode = nodes.at(i).toElement();
		ID 				= cNode.attribute("ID").toInt();
		custID 			= cNode.attribute("customID").toInt();
		groupID 		= cNode.attribute( "groupID" ).toInt();
		layer			= cNode.attribute( "layer" ).toInt();
		nbPoints 		= cNode.attribute( "nbPoints" ).toInt();
		dType 			= cNode.attribute( "drawType" ).toInt();
		angle			= cNode.attribute( "angle" ).toFloat();
		name 			= cNode.attribute( "name" );
		act 			= cNode.attribute( "active" ).startsWith("1") ; // true if 1 
		transparency 	= cNode.attribute( "transparency" );
		texture_path 	= cNode.attribute( "texture_path" );
		shape 			= cNode.attribute( "shape" );
		realMin 		= cNode.attribute( "realMin" ).toFloat();
		realMax 		= cNode.attribute( "realMax" ).toFloat();
		scale			= cNode.attribute( "scale" ).toFloat();		
		spatialPeriod	= cNode.attribute( "spatialPeriod" ).toInt();		
		host			= cNode.attribute( "host" ) ;	
		port			= cNode.attribute( "port" ).toInt() ;	
		
		// Color
		Color col1 = parseColors(cNode).at(1) ; 
		Color col2 = parseColors(cNode).at(0) ;
		
		// Position
		QDomNodeList pnodes = cNode.elementsByTagName("Position") ;
		QDomElement el = pnodes.at(0).toElement() ;
		curvePos.ID = el.attribute("ID").toInt() ;
	  	curvePos.x 	= el.attribute("x").toFloat() ;
		curvePos.y 	= el.attribute("y").toFloat() ;
	  	curvePos.z	= el.attribute("z").toFloat() ;	
		
		// Curve definition
		points = parsePoints(cNode);
				
		// Adding new Curve
		NxCurve *curve = new NxCurve( ID, custID, groupID, layer, name, act, curvePos, col1, points, points.count(), col2, 1, texture_path, shape, realMin, realMax, scale, angle, spatialPeriod, host, port );
		if(dType == SplineStrip) curve->splineGenerate() ;							  
		curve->setDrawType( dType );
		curve->setAngle( angle ) ;
		curves.insert(ID, curve);
	}
	return curves;
}

// ======================
// ### PARSE TRAJECTS ###
// ======================
QMap<int, NxTraject*> NxParser::parseTrajects()
{
	int 				ID, custID, groupID, layer, nbCursors, type, orient ;
	bool				act ;
  	QList<int> 			cursorIDs;
	QString				name;
  	QList<Color> 		colors;
  	QVector<NxPoint>	curveDef;
  	Circle 				c;
	c.radius = c.p.ID = 0 ;		// initialisation
	c.p.x = c.p.y = c.p.z = 0 ; // initialisation
	
  	// Parsing NxCurve nodes
	QDomNodeList nodes = document.elementsByTagName("NxTraject");
  	
	for (int i=0; i<nodes.count(); i++) 
	{
		QDomElement cNode = nodes.at(i).toElement();

		ID 			= cNode.attribute("ID").toInt();
		custID 		= cNode.attribute("customID").toInt();
		groupID 	= cNode.attribute("groupID").toInt();
		layer		= cNode.attribute("layer").toInt();
		nbCursors 	= cNode.attribute("nbCursors").toInt();
		name 		= cNode.attribute("name");
		act 		= cNode.attribute("active").startsWith("1") ;

		// Parsing curve def : line, circle, or B-spline
		cursorIDs 	= parseCursorID( cNode, nbCursors );
		
		// Color
		Color col = parseColors(cNode).at(0) ;	
		
		type = parseShape(cNode);
		
		if (type == 1 ) 
			c = parseCircle(cNode);
		
		// Parsing cursors & traject definition
		NxTraject *traject = new NxTraject( ID, custID, groupID, layer, name, act, c.p, col, type, nbCursors, cursorIDs );
				
		// Adding shape information
		QDomNodeList nod = cNode.elementsByTagName("Orientation") ;
		QDomElement elem = nod.at(0).toElement();
		orient = elem.attribute("type").toInt();
		
		curveDef = parsePoints(cNode);
		
		switch (type) {
			case 1:	// Circle
	  			traject->setCircleDef( c );
	  			traject->evalBoundingBox( c.p , 1.0);
	  		break;
			case 2:	// Line
	  			traject->setLineDef( curveDef[0], curveDef[1] );
	  			traject->setPosition( curveDef[0] );
	  			traject->evalBoundingBox( curveDef[0], 1.0 );
				traject->setOrientation( orient ) ;
	  		break;
		}

		//printf("Inserting ID %i in Trajects\n", ID);
		trajects.insert(ID, traject);		
	}
	return trajects;
}


// =====================
// ### PARSE CURSORS ###
// =====================
QMap<int, NxCursor*> NxParser::parseCursors()
{
	// Cursors data
  	int 			ID,custID, groupID, layer, parentID;
  	float 			offset, initoffset, finaloffset, pFactor, newWidth;
 	QString			name ;
	bool			inForward ;
	bool			act, playF ;
	int				inMode ;
  	State 			*widthLine, *speedLine;
  	
	QVector<NxPoint> points;
  	NxPoint startPt;
	
	// Parsing
	QDomNodeList nodes = document.elementsByTagName("NxCursor");
  	
	for(int i=0; i<nodes.count(); i++)
	{
		QDomElement elem = nodes.at(i).toElement();
		
		ID 			= elem.attribute("ID").toInt();
		custID		= elem.attribute("customID").toInt();
		groupID 	= elem.attribute("groupID").toInt();
		layer		= elem.attribute("layer").toInt();
		parentID 	= elem.attribute("parentID").toInt();
		name 		= elem.attribute("name") ;
		act 		= elem.attribute("active").startsWith("1") ;
		playF		= elem.attribute("playFinished").startsWith("1") ;
		inForward	= elem.attribute("forward").startsWith("1") ;
		inMode 		= elem.attribute("mode").toInt() ;
		offset 		= elem.attribute("offset").toFloat();
		initoffset	= elem.attribute("init_offset").toFloat();
		finaloffset = elem.attribute("final_offset").toFloat();
		newWidth 	= elem.attribute("width").toFloat();
	
		// Color
		Color col = parseColors(elem).at(0) ;
		
		// Extracting starting NxPoint
		points = parsePoints(elem);
		startPt = points[0];

		// Extracting factor
		pFactor = parseFactor(elem);
			
		// Extracting width timeline
		int nbSpeed, nbWidth;
		//speedLine	= parseTimeLine(elem, "speed", &nbSpeed );
		// Extracting speed timeline
		//widthLine	= parseTimeLine(elem, "width", &nbWidth ); 
			
		//printf("Parent trajectory : %i\n", parentID);
		//printf("Trajectory ID : %i\n", trajects.value(parentID)->getID());

		// Adding new cursor
		NxCursor *cursor = new NxCursor( ID, custID, groupID, layer, trajects.value(parentID), name, act, startPt, col, playF, inForward, inMode, 
							   speedLine, widthLine, nbSpeed, nbWidth, newWidth, initoffset, finaloffset );
		cursor->setOffset( offset );
		//cursor->setInitOffset( offset ); // done...
		cursor->setSpeed( pFactor );
	
		cursors.insert(ID, cursor);
	}
  	return cursors;
}


// ======================
// ### PARSE TRIGGERS ###
// ======================
QMap<int, NxTrigger*> NxParser::parseTriggers()
{
	// Triggers data
  	int 				ID,custID, groupID, layer, port, inIntValue;
	bool				act ;
  	QString				name, host;
  	QVector<NxPoint> 	points;
  	NxPoint 			startPt;
			
	QDomNodeList triggernodes = document.elementsByTagName("NxTrigger");
	
	if (triggernodes.isEmpty())
		return triggers;
	
	// Parsing
  	for (int i=0; i<triggernodes.count(); i++) 
	{
		QDomElement cNode = triggernodes.at(i).toElement();
		
		ID 			= cNode.attribute("ID").toInt();
		custID 		= cNode.attribute("customID").toInt();
		groupID 	= cNode.attribute("groupID").toInt();
		layer		= cNode.attribute("layer").toInt();
		port 		= cNode.attribute("port").toInt();
		name 		= cNode.attribute("name");
		act 		= cNode.attribute("active").startsWith("1") ;
		host 		= cNode.attribute("host");
		inIntValue	= cNode.attribute("value").toInt();
		
		// Color
		Color col = parseColors(cNode).at(0) ;
		
		// Position parsing
		points = parsePoints(cNode);
		startPt = points[0];
		startPt.ID = 0 ;
		
		NxTrigger *trigger = new NxTrigger( ID, custID, groupID, layer, name, act, startPt, col, host, port, inIntValue ) ;
		triggers.insert(ID, trigger);
	}
	return triggers;
}


// =================
// ### PARSE ENV ###
// =================
NxEnv NxParser::parseEnv()
{
  	NxEnv 	newEnv;
  	QString	name ;
  	QString UDPhost, OSCCurvesDefaultMFormat, OSCTriggersDefaultMFormat ;
	QString oscCurvesF, oscTriggersF ;
	bool scCGID, scCID, scGID, scID, scA, scV ;
	bool stCGID, stCID, stGID, stID, stV ;
	int		OSCTriggersDefaultIntValue ;
	float	mappingDefaultMin, mappingDefaultMax, zoom, gridStep ;
  	int 	ID, length, height;
  	int 	inPort, outPort;
  	int 	nbCurves, nbTrajects, nbTriggers, nbCursors;
	int		disStep, schStep, defaultSpatialPeriod ;
  	QStringList comments ;
	float globalTempo, startTime, stopTime ;
	Color c1, c2, c3 ;
	bool visualMode, oscInMode, oscOutMode, sendIFmessages ;
	bool texturing, transparency, background ;
	bool useCirclesAlgo, crossesDraw, scoreDraw, displayOn, showGrid, snapGrid, autoGridStep ;
	bool circlesDraw, rectDraw, showCursorsArea, broadcastMode ;
	bool showTrajects, showCurves, showCursors, showTriggers ;
	QString xmlVersionOld, xmlVersionNew, xmlVersion ;
	float xmlVersionNumber ;
	
	// NxEnv Tag
	QDomNodeList nodes = document.elementsByTagName("NxEnv");

	if (nodes.isEmpty())
		return newEnv;
	
	QDomElement root = nodes.at(0).toElement();
	
	QDomElement child;

  	// XML Version
  	child			= root.elementsByTagName("Version").at(0).toElement();	
	xmlVersionOld 	= child.attribute("v");
	child			= root.elementsByTagName("XMLVersion").at(0).toElement();	
	xmlVersionNew 	= child.attribute("v");
	
	if(xmlVersionOld.contains("0.6")) xmlVersion = xmlVersionOld ;
	else xmlVersion = xmlVersionNew ;
	
	QString xmlTmpVersion = xmlVersion ;
	xmlVersionNumber = xmlTmpVersion.remove("b").toFloat() ;
	printf("XML Version Number Parsed: %f\n", xmlVersionNumber) ;
	
  	// General settings
  	child 	= root.elementsByTagName("ID").at(0).toElement();	
	name 	= child.attribute("name");
  	ID 		= child.attribute("ID").toInt();
	visualMode 	= child.attribute("visualMode")=="1" ;
	oscOutMode 	= child.attribute("oscOutMode")=="1" ;
	oscInMode 	= child.attribute("oscInMode")=="1" ;

  	// Bounds settings
  	child	= root.elementsByTagName("Bounds").at(0).toElement();  	
	length 	= child.attribute("length").toInt();
  	height 	= child.attribute("height").toInt();

  	// Objects Stats
  	child		= root.elementsByTagName("Stats").at(0).toElement();
	nbCurves 	= child.attribute("nbCurves").toInt();
  	nbTrajects 	= child.attribute("nbTrajects").toInt();
  	nbTriggers 	= child.attribute("nbTriggers").toInt();
  	nbCursors 	= child.attribute("nbCursors").toInt();
	
	// Time settings
  	child 		= root.elementsByTagName("Time").at(0).toElement();	
	useCirclesAlgo	= child.attribute("circlesAlgo")=="1" ;
	globalTempo = child.attribute("globalTempo").toFloat() ;
	startTime	= child.attribute("startTime").toFloat() ;
	stopTime = child.attribute("stopTime").toFloat() ;
  	disStep = child.attribute("displayStep").toInt() ;
	schStep = child.attribute("schedulerStep").toInt() ;
	
	// Network settings
  	child 		= root.elementsByTagName("Network").at(0).toElement();	
	UDPhost 	= child.attribute("host") ;
  	inPort		= child.attribute("inPort").toInt();
  	outPort 	= child.attribute("outPort").toInt();
	broadcastMode				= child.attribute("broadcastMode")=="1" ;
	OSCCurvesDefaultMFormat 	= child.attribute("curvesOSC") ;
	OSCTriggersDefaultMFormat	= child.attribute("triggersOSC") ;
	if(xmlVersionNumber>=0.6385) { // IFmessages + OSC Messages Format parsing (FROM 0.639 INCLUDED)
		sendIFmessages	= child.attribute("sendIFmessages")=="1" ;
		oscCurvesF		= child.attribute("curvesArgs") ;
		oscTriggersF	= child.attribute("triggersArgs") ;
		scCGID	= oscCurvesF.at(0)==QChar('1') ;
		scCID	= oscCurvesF.at(1)==QChar('1') ;
		scGID	= oscCurvesF.at(2)==QChar('1') ;
		scID	= oscCurvesF.at(3)==QChar('1') ;
		scA		= oscCurvesF.at(4)==QChar('1') ;
		scV		= oscCurvesF.at(5)==QChar('1') ;
		stCGID	= oscTriggersF.at(0)==QChar('1') ;
		stCID	= oscTriggersF.at(1)==QChar('1') ;
		stGID	= oscTriggersF.at(2)==QChar('1') ;
		stID	= oscTriggersF.at(3)==QChar('1') ;
		stV		= oscTriggersF.at(4)==QChar('1') ;
	}
	else { // Default OSC Format (we send all)
		sendIFmessages = false ;
		scCID = scCGID = scID = scGID = scA = scV = true ;
		stCID = stCGID = stID = stGID = stV = true ;
	}
	
	// Display
  	child		= root.elementsByTagName("Display").at(0).toElement();	
	zoom		= child.attribute("zoom").toFloat();
  	scoreDraw 	= child.attribute("scoreDraw")=="1" ;
	displayOn 	= child.attribute("displayOn")=="1" ;
	crossesDraw = child.attribute("crossesDraw")=="1" ;
	
	// Color settings
	child 	= root.elementsByTagName("BackgroundColor").at(0).toElement();	
	c1.ID = child.attribute("ID").toInt() ;
	c1.c.setRed( child.attribute("r").toInt() ) ;
	c1.c.setGreen( child.attribute("g").toInt() ) ;
	c1.c.setBlue( child.attribute("b").toInt() ) ;
	c1.c.setAlpha( child.attribute("a").toInt() ) ;
	child 	= root.elementsByTagName("ScoreColor").at(0).toElement();	
	c2.ID = child.attribute("ID").toInt() ;
	c2.c.setRed( child.attribute("r").toInt() ) ;
	c2.c.setGreen( child.attribute("g").toInt() ) ;
	c2.c.setBlue( child.attribute("b").toInt() ) ;
	c2.c.setAlpha( child.attribute("a").toInt() ) ;
	if(xmlVersionNumber>=0.6375) { // Grid Color (FROM 0.638 INCLUDED)
		child 	= root.elementsByTagName("GridColor").at(0).toElement();	
		c3.ID = child.attribute("ID").toInt() ;
		c3.c.setRed( child.attribute("r").toInt() ) ;
		c3.c.setGreen( child.attribute("g").toInt() ) ;
		c3.c.setBlue( child.attribute("b").toInt() ) ;
		c3.c.setAlpha( child.attribute("a").toInt() ) ;
	}
	else { // Default Grid Color
		c3.ID = 0 ;
		c3.c = QColor(255,255,255,255) ;
	}
	
  	// Render settings
  	child			= root.elementsByTagName("Render").at(0).toElement() ;  	
	texturing 		= child.attribute("texturing")=="1" ;
  	transparency 	= child.attribute("transparency")=="1" ;
  	background 		= child.attribute("background")=="1" ;

  	// Grid
  	child			= root.elementsByTagName("Grid").at(0).toElement();	
	showGrid		= child.attribute("showGrid")=="1" ;
	snapGrid		= child.attribute("snapGrid")=="1" ;
	autoGridStep 	= child.attribute("autoGridStep")=="1" ;
	gridStep		= child.attribute("gridStep").toFloat();

  	///////////////////// TRAJECTS /////////////////////
  	child				= root.elementsByTagName("Trajects").at(0).toElement();	
	showTrajects	 	= child.attribute("show")=="1" ;
	
  	///////////////////// CURSORS /////////////////////
  	child				= root.elementsByTagName("Cursors").at(0).toElement();	
  	showCursors			= child.attribute("show")=="1" ;
	showCursorsArea 	= child.attribute("showCursorsArea")=="1" ;
		
	///////////////////// CURVES /////////////////////
  	child 					= root.elementsByTagName("Curves").at(0).toElement();	
  	showCurves				= child.attribute("show")=="1" ;
	circlesDraw				= child.attribute("circlesDraw")=="1" ;
  	rectDraw				= child.attribute("rectDraw")=="1" ;
	mappingDefaultMin		= child.attribute("mappingDefaultMin").toFloat();
	mappingDefaultMax		= child.attribute("mappingDefaultMax").toFloat();
	if(xmlVersionNumber<=0.6355) 
		defaultSpatialPeriod = child.attribute("defaultSpatialPeriod").toInt();
	else // (FROM 0.636 INCLUDED)
		defaultSpatialPeriod = child.attribute("defaultScanPeriod").toInt();
	
  	///////////////////// TRIGGERS /////////////////////
  	child 						= root.elementsByTagName("Triggers").at(0).toElement();	
  	showTriggers 				= child.attribute("show")=="1" ;
	OSCTriggersDefaultIntValue 	= child.attribute("defaultIntValue").toInt();
	
  	// Comments (through loop)
  	QDomNodeList cnodes = root.elementsByTagName("comment");
  	
	for (int i=0;i<cnodes.count();i++) {
		comments << cnodes.at(i).toElement().attribute("str");
	}
	
  	///////// GENERAL
  	newEnv.xmlVersion	= xmlVersion ;
  	newEnv.name 		= name;
  	newEnv.ID 			= ID;
  	newEnv.groupID 		= 0 ;
  	newEnv.length		= length;
  	newEnv.height		= height;
  	newEnv.nbCurves 	= nbCurves;
  	newEnv.nbTrajects 	= nbTrajects;
  	newEnv.nbTriggers 	= nbTriggers;
  	newEnv.nbCursors 	= nbCursors;
  	newEnv.nbComm 		= cnodes.count();
  	newEnv.comments 	= comments ;
  	///////// MODES
  	newEnv.visualMode	= visualMode ;
	newEnv.oscOutMode	= oscOutMode ;
	newEnv.oscInMode	= oscInMode ;
	newEnv.showTrajects	= showTrajects ;
	newEnv.showCursors	= showCursors ;
	newEnv.showCurves	= showCurves ;
	newEnv.showTriggers	= showTriggers ;
  	///////// OBJECT DEFAULT VALUES
  	newEnv.defaultSpatialPeriod 		= defaultSpatialPeriod ;
  	newEnv.OSCTriggersDefaultIntValue	= OSCTriggersDefaultIntValue ;
	newEnv.mappingDefaultMin 			= mappingDefaultMin ;
	newEnv.mappingDefaultMax 			= mappingDefaultMax ;
	///////// FPS
	newEnv.displayStep		= disStep ;
	newEnv.schedulerStep	= schStep ;
	///////// DISPLAY
	newEnv.backColor	= c1 ;
	newEnv.scoreColor	= c2 ;
	newEnv.gridColor	= c3 ;
	newEnv.zoom				= zoom ;
	newEnv.showGrid			= showGrid ;
	newEnv.crossesDraw		= crossesDraw ;
	newEnv.scoreDraw		= scoreDraw ;
	newEnv.displayOn		= displayOn ;
	newEnv.circlesDraw		= circlesDraw ;
	newEnv.rectDraw			= rectDraw ;
	newEnv.snapGrid			= snapGrid ;
	newEnv.autoGridStep		= autoGridStep ;
	newEnv.gridStep			= gridStep ;
	newEnv.showCursorsArea	= showCursorsArea ;
	///////// TIME
	newEnv.useCirclesAlgo		= useCirclesAlgo ;
	newEnv.globalTempo			= globalTempo ;
	newEnv.initialTime			= startTime ;
	newEnv.finalTime			= stopTime ;
	///////// NETWORK
	newEnv.broadcastMode	= broadcastMode ;
	newEnv.sendIFmessages	= sendIFmessages ;
  	newEnv.UDPhost 			= UDPhost ;
  	newEnv.UDPout 			= outPort ;
  	newEnv.UDPin			= inPort ;
	newEnv.OSCCurvesDefaultMFormat		= OSCCurvesDefaultMFormat ;
	newEnv.OSCTriggersDefaultMFormat	= OSCTriggersDefaultMFormat ;
	newEnv.scCGID	= scCGID ;
	newEnv.scCID	= scCID ;
	newEnv.scGID	= scGID ;
	newEnv.scID		= scID ;
	newEnv.scA		= scA ;
	newEnv.scV		= scV ;
	newEnv.stCGID	= stCGID ;
	newEnv.stCID	= stCID ;
	newEnv.stGID	= stGID ;
	newEnv.stID		= stID ;
	newEnv.stV		= stV ;
	newEnv.OSCConfigMessageIP			= QString("255.255.255.255") ;
	
  	return newEnv;
}

// ===================
// ### SUB PARSERS ###
// ===================


// ====================
// ### PARSE FACTOR ###
// ====================
float NxParser::parseFactor( const QDomElement &elem )
{
	QDomNodeList results = elem.elementsByTagName("factor");
	
	if (results.isEmpty())
		return 0.0;
	
	QDomElement fNode = elem.elementsByTagName("factor").at(0).toElement();
	
  	return fNode.attribute("coeff").toFloat();
}


// ======================
// ### PARSE TIMELINE ###
// ======================
State* NxParser::parseTimeLine( const QDomElement &elem, char* inStr, int* index )
{
  	// Init TimeLine
  	State	*timeline = new State[*index];
	State 	tmpState;
	
	QDomNodeList speeds = elem.elementsByTagName("speed");
	QDomNodeList widths = elem.elementsByTagName("width");
	
  	for (int i=0; i<speeds.count();i++)
 	{	
	  	//tagAttributes = childNode->getAttributes();
	  	//tmpState.type = parseString( "type" );
		//tmpState.duration = parseInteger( "duration" );
	  	//tmpState.frequency = parseInteger( "frequency" );
	  	//tmpState.coeff = parseFloat( "coeff" );		
	  	//tmpState.scale = parseFloat( "scale" );
	  	//timeline[i] = tmpState;
	}
	return timeline;
}


// ===================
// ### PARSE SHAPE ###
// ===================
int NxParser::parseShape(const QDomElement &elem)
{
	QDomNodeList results = elem.elementsByTagName("shape");
	
	if (results.isEmpty())
		return 0;
	
	QDomElement fNode = elem.elementsByTagName("shape").at(0).toElement();
	
  	return fNode.attribute("type").toInt();
}


// ====================
// ### PARSE POINTS ###
// ====================
QVector<NxPoint> NxParser::parsePoints(const QDomElement &elem)
{
 	QVector<NxPoint> 	points;
	QDomNodeList 		nodes = elem.elementsByTagName("NxPoint");
	
	for (int i=0; i<nodes.count(); i++) 
	{	
		QDomElement el = nodes.at(i).toElement();
		NxPoint 	pt;
	  	
		pt.ID 	= el.attribute("ID").toInt();
	  	pt.x 	= el.attribute("x").toFloat();
		pt.y 	= el.attribute("y").toFloat();
	  	pt.z	= el.attribute("z").toFloat();	
	  	points.append(pt);
	}
	return points;			
}


// ====================
// ### PARSE COLORS ###
// ====================
QList<Color> NxParser::parseColors(const QDomElement &elem)
{
	QList<Color> colors;
	QDomNodeList nodes = elem.elementsByTagName("color");

  	for (int i=0; i<nodes.count(); i++) 
	{	
	  	QDomElement	el = nodes.at(i).toElement();
		Color clr;
		
	  	clr.ID 	= el.attribute("ID").toInt();
		QColor col = QColor( el.attribute("r").toInt() , el.attribute("g").toInt() , el.attribute("b").toInt() , el.attribute("a").toInt() ) ;
	  	clr.c = col ;
	
		colors << clr;
	}
	return colors;			
}


// =======================
// ### PARSE CURSOR ID ###
// =======================
QList<int> NxParser::parseCursorID( const QDomElement &elem, int inNbCursors )
{
  	QList<int> IDs;
  	QDomNodeList nodes = elem.elementsByTagName("NxCursor");
	
	for ( int i=0; i<nodes.count(); i++)
		IDs << nodes.at(i).toElement().attribute("ID").toInt();
  	
	return IDs;
}


// ====================
// ### PARSE CIRCLE ###
// ====================
Circle NxParser::parseCircle( const QDomElement &elem )
{
	Circle c;
  	QDomNodeList nodes = elem.elementsByTagName("circle");
	
	if (nodes.isEmpty())
		return c;
	
	QDomElement el = nodes.at(0).toElement();
	  		
	c.radius 	= el.attribute("radius").toFloat();
	c.p.ID 		= el.attribute("ID").toInt();
	c.p.x 		= el.attribute("x").toFloat();
	c.p.y 		= el.attribute("y").toFloat();
	c.p.z 		= el.attribute("z").toFloat();
  	
	return c;
}

